#!/bin/bash
#
# Usage examples:
#
#   mjc.sh tertium
#   mjc.sh -p tertium
#   mjc.sh
#   mjc.sh -p



SWIPL=swipl   # The Prolog interpreter 
MJC=mjc.pl    # launcher file

Options=""

# Print the usage of this script
function usage()
{
    cat << EOF
Usage: ${0##*/}  [OPTION] .... [FORMULA_LABEL]
       
Prove the formula having label FORMULA_LABEL defined in the test file.
If no label is specified as argument, prove all the formulas defined in the test file.
The default test file is test.pl.

Options:
  -f <file>    set the specifie file as test file
  -h           show this message  
  -p           run pdflatex to generate a pdf file
  -s           show the command line used to launch the Prolog application

Examples:
   mjc.sh tertium
   mjc.sh -p tertium
   mjc.sh
   mjc.sh -p

EOF
}



# PARSE OPTIONS IN COMMAND LINE


while getopts  ":f:hps" option
#  A ';'  must be placed after an option with an  argument
# First ';' sets silent mode 
do
  case $option in  
      f ) # set the test file 
	  test_file=$OPTARG
          if [ ! -r $test_file ]; then
	      echo "$test_file: no such file"
	      exit 1
	  fi
          ;;  
      h ) # help
	  usage
          exit 1
	  ;;
      p ) # generate pdf
	  pdf=1
	  ;;
      s ) # show the command line
	  show_cmd=1
	  ;;
      \?) # error
          echo "Invalid option" >&2
          exit -1
          ;;
  esac  
done 

#shift positional parameters

shift $(($OPTIND - 1)) 



####  MAIN


form_label=$1
prolog_args=""



if [ -z  "$form_label" ]; then  # no formula has been specified
# prove all the defined formulas 
    if [  $pdf ]; then
	# pdf
	if [ $test_file ]; then
	    prolog_args=$prolog_args" -g provepdf_all('$test_file')"
	    else
	    prolog_args=$prolog_args" -g provepdf_all"
	fi
    else
	# no pdf
	if [ $test_file ]; then
	    prolog_args=$prolog_args" -g prove_all('$test_file')"
	    else
	    prolog_args=$prolog_args" -g prove_all"
	fi
    fi
else
# only prove $form_label
    if [  $pdf ]; then
	# pdf
	if [ $test_file ]; then
	    prolog_args=$prolog_args" -g provepdf('$form_label','$test_file')"
	else
	    prolog_args=$prolog_args" -g provepdf('$form_label')"
	fi
    else
	# no pdf
	if [ $test_file ]; then
	    prolog_args=$prolog_args" -g prove('$form_label','$test_file')"
	else
	    prolog_args=$prolog_args" -g prove('$form_label')"
	fi
    fi
fi

# build cmd_line

cmd_line="$SWIPL --quiet -f $MJC "$prolog_args" -t halt"

if [  $show_cmd ]; then # print cmd_line
    echo  $cmd_line
fi

# run cmd_line
    
$cmd_line


