%################### ELIMINATION SUBSTITUTIONS IN A MJC-DERIVATION
% #################



:- module(substElim,
	  [
	      subst_elim/2
	  ]).

:- use_module(syntax).


:- use_module(basic,[
		  weak_tree_set/3,
		  replace_leaves/3
	      ]).


%
%   *  A pattern
%
%             Gamma ; H '|>'
%
%    is represented by the Prolog term
%
%           Gamma @ H '|>'
%
%
%  * A ptree ia an MJC-tree of the form
%
%     { Gamma @ foc(A) '|>' }
%		.
%		.
%	        .
%       Gamma @ foc(H)  '|>'
%
%   where:
%    -   the pattern Gamma @ foc(H)  '|>' is the root and the pattern
%    -   Gamma @ foc(A) '|>'  is the only open leaf.
%    -   the displayed branch only contains patterns
%
%     The above ptree is represented  in normal form by the term
%
%
%      PTree ::: A
%
%     where A is the focused formula in the open leaf of PTree
%
%
%   * The premises  of a substitution rule are called:
%
%    - main premise:  the MJC-derivation
%    - side premises: the ptress
%
%



% get_main_and_sideNF_premises_subst(+Prems, -MainPrem, -SidePremsNF,-Theta)
%
% * Prems:  the premises of a subst application *
% * MainPrem: the main premise of subst
% * SidePremsNF: the side premises in normal form
% * Theta = { A | Ptree ::: A \in SidePremsNF }

get_main_and_sideNF_premises_subst([],_,[],[]):- !.

get_main_and_sideNF_premises_subst( [proof(Rule, Gamma ==> K, Prs)  | Trees ], MainPrem	, SidePremsNF, Theta) :- !,
	MainPrem = proof(Rule, Gamma ==> K, Prs),
	get_main_and_sideNF_premises_subst( Trees,MainPrem, SidePremsNF,Theta).

get_main_and_sideNF_premises_subst( [ PTree  |Trees ], MainPrem,SidePremsNF,Theta) :-
	assertion( PTree =  proof(_Rule,_Gamma @ _  '|>'  ,_) ),
	get_top_focused_formula(PTree ,A),
	get_main_and_sideNF_premises_subst( Trees,MainPrem,  SidePremsNF1,Theta1),
	SidePremsNF =  [  PTree ::: A  | SidePremsNF1],
	Theta = [ A | Theta1].


% get_top_focused_formula(+PTree, -A)
%
% Get the focused formula in the open leaf of the ptree PTree

get_top_focused_formula(proof(leaf,_Gamma @ foc(A) '|>'  , []),A) :- !.
get_top_focused_formula(proof(_Rule,_Gamma @ foc(_A & _B) '|>'  , [P0]),Top):- !,
	get_top_focused_formula(P0,Top).

get_top_focused_formula(proof(_Rule,_Gamma @ foc(_A --> _B) '|>'  , [_P0,P1]),Top) :-
	get_top_focused_formula(P1,Top).


%%%%%%  SUBSTITUTION  ELIMINATION %%%%%
%
% subst_elim(+MJC_Proof, -MJC_Proof_out)
%
% MJC_Proof_out is obtained from MJC_Proof
%  by eliminating all the applications of subst rule.
%

subst_elim(proof(subst, _Gamma ==>  _K ,Premises),Proof_out) :- !,
         %% eliminate subst applications in all the premises
	 subst_elim_list(Premises,Premises_s),
	 get_main_and_sideNF_premises_subst(Premises_s,MainPrem_s,PTreesNF_s,Theta),
	 PTreesNF_s = [ proof(_, _ @ foc(H) '|>' , _Prems ) ::: _| _OtherPTrees], %% extract H
	 apply_subst(MainPrem_s, Theta,H,PTreesNF_s,Proof_out).


subst_elim(proof(Rule, RootSeq ,Prems),Proof_out) :-
    assertion(Rule \= subst),
    subst_elim_list(Prems,Prems1),
    Proof_out = proof(Rule,RootSeq,Prems1).


subst_elim_list([],[]).
subst_elim_list([Proof | Proofs], [Proof1 | Proofs1]) :-
	subst_elim(Proof,Proof1),
	subst_elim_list(Proofs, Proofs1).



is_leftContrRule(Rule) :- member(Rule,[lc0,lc1]).

%% apply_subst(+MJC_Proof,+Theta,+H,+PTreesNF,-MJC_Proof_out)
%
% Apply substitution of MJC_proof
% Theta must be replaced eith H
% PTreesNF are the required ptrees in normal form
%

apply_subst(proof(LCRule, Gamma ==>  K, [P0]), Theta, H,PTreesNF, Proof_out) :-
	is_leftContrRule(LCRule), !,
	P0 = proof(_, _ @ foc(A) =*=> lab(L) @ _, _Prems), % extract the focused formula A and Lab
	replace(Theta,H,Gamma,Gamma_min_Theta_U_H),
	(
	    member(A,Theta) ->
	    %%% crucial case
	     apply_subst(P0,Theta,H,PTreesNF, P0_s),
	     %%%%  choose the ptree PTreeA to be closed and insert it in the right way
	     member(PTreeA :::  A, PTreesNF),
	     PTreeA = proof(_, GammaA @ _ '|>' ,_), %% extract GammaA
	     fill_holes(PTreeA, L, K, TreeA),
	     subtract(Gamma_min_Theta_U_H, GammaA, Delta),
	    %% Delta = Gamma_min_Theta_U_H  \ GammaA
	     weak_tree_set(TreeA,Delta,WTreeA), % add Delta to the contexts in TreeA
	     WTreeA_LC = proof(LCRule, Gamma_min_Theta_U_H ==>  K, [WTreeA]),
	     replace_leaves(P0_s ,WTreeA_LC,Proof_out)
	     % replace the open leaf of WTreeA_LC with P0_s
	   ;
	   % A is not in Theta, non-crucial case
	    apply_subst(P0,Theta,H,PTreesNF,P0_s),
	    Proof_out = proof(LCRule, Gamma_min_Theta_U_H ==> K, [P0_s])
	).
% other  non-crucial cases
apply_subst(proof(Rule, Gamma ==>  K,Prems), Theta,H, PTreesNF, Proof_out) :- !,
	assertion( \+ is_leftContrRule(Rule)),
	replace(Theta,H,Gamma,Gamma_min_Theta_U_H),
	apply_subst_list(Prems, Theta,H, PTreesNF,Prems_s),
	Proof_out = proof(Rule, Gamma_min_Theta_U_H ==> K, Prems_s).

apply_subst(proof(Rule, Gamma @ foc(A) =*=> lab(L) @ K,Prems), Theta,H, PTreesNF, Proof_out) :- !,
	replace(Theta,H,Gamma,Gamma_min_Theta_U_H),
	apply_subst_list(Prems, Theta,H, PTreesNF,Prems_s),
	Proof_out = proof(Rule, Gamma_min_Theta_U_H @ foc(A) =*=> lab(L) @ K,Prems_s).

apply_subst(proof(_Rule, _Gamma @ _ '|>',_Prems), _Theta,_H, _PTreesNF, _Proof_out) :-
	writeln('XXXXXXXXXXXXXXXXXXXXX'),abort.

apply_subst_list([],  _Theta, _H, _PTreesNF, []).
apply_subst_list([Proof|Proofs], Theta, H,PTreesNF, [Proof_s| Proofs_s ] ):-
	apply_subst(Proof,Theta,H,PTreesNF,Proof_s),
	apply_subst_list(Proofs, Theta, H,PTreesNF, Proofs_s  ).

% replace(+Theta,+H,+Gamma, -Gamma_min_Theta_U_H)
% Gamma_min_Theta_U_H = (Gamma \ Theta) U {H}

replace(Theta,H,Gamma, Gamma_min_Theta_U_H) :-
	subtract(Gamma,Theta,Gamma_min_Theta),
	union([H], Gamma_min_Theta, Gamma_min_Theta_U_H).


% fill_holes(+PTree,+Label, +K , -MJC_Proof).
% MJC_Proof is obtained by filling the holes of the patterns in PTrees

fill_holes(proof(Rule, Gamma @ foc(H) '|>' , Prems), Lab, K , Proof_out) :-!,
      fill_holes_list(Prems, Lab,K, Prems1),
      Proof_out = proof(Rule, Gamma @ foc(H) =*=> lab(Lab) @ K , Prems1).

fill_holes( Proof, _Lab, _K , Proof).


fill_holes_list([],_,_,[]).
fill_holes_list([Proof |Proofs] , Lab, K, [Proof1|Proofs1] ) :-
	   fill_holes(Proof,Lab,K,Proof1),
	   fill_holes_list(Proofs, Lab, K, Proofs1 ).




















