%%%%%    FORMULA DEFINITIONS     %%%%%%%
%
%   ### Default test file   ####
%


%#############    DO NOT DELETE  ###################################
:- use_module(syntax).
:-  op(300, xfy, '->').  %%% only used  to detect syntax errors
%#####################################################################


% ##### SET THE DIRECTORY FOR THE LATEX AND PDF FILES (TEST DIR) #######



test_dir(examples). %   dir. examples in the current dir.

/** Other possible definition of test_dir  **/
/*
test_dir(TestDir) :-   %   dir. examples in my HOME
       getenv('HOME', HomeDir),
       directory_file_path(HomeDir,'examples',TestDir).

*/



%#####  SYNTAX FOR FORMULA DEFINITIONS  ########

/*

* Prolog atom ::= any identifier starting with a lower-case letter.
	          Identifiers beginning with a capital must  be quoted.

Examples of Prolog atoms:

    p       a1     q_3       'F'      'F_2'

Prolog atoms are used to represent propositional variables.

* Formula F ::= Prolog_atom | #f | ~F | F & F | F v F | F --> F

  #f represents falsum, ~F is an abbreviation for F --> #f


Examples of formulas:

   (p&q)--> ~p v ~q	   (p_0-->q)& (p_1-->q)-->p_0 v p_1-->q

    'A' --> a v 'B_0' v 'B_1'


NOTE
^^^^

- OR is 'v' (and not '|'), IMPLIES is '-->' (and not'->').
- in the latter example. 'A' and a are distinct variables.


* Formula definition

A formula is defined by one between the predicates

1)  test(FormLabel,Formula)
2)  test(FormLabel,Formula,Defs)

In both cases, FormLabel (a Prolog atom) is a label used to identify
the formula.



1)  test(FormLabel,Form)
^^^^^^^^^^^^^^^^^^^^^^^^

It defines a formula Form  with label FormLabel


2)  test(FormLabel,Form,Defs)
^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

It defines a formula Form with label FormLabel using a list Defs of
auxiliary definitions to designate subformulas of Form.
Definitions are *only* used to pretty-print derivations.

Defs is a Prolog list of the form

 [ at_1 = f_1 , ....   , at_n = f_n ]

 where:

 -  at_1  ... at_n  (defined atoms) are Prolog_atoms;
 -  f_1   ... f_n    are formulas.

The formulas Form,  f_1, ... , f_n may contain defined atoms.

For instance, given the definitions

 [ a_0 = p_0-->q , a_1 = p_1-->q , a = a_0 & a_1 , b =  p_0 v p_1 ]

the formula

a --> b --> q

is an abbreviation for

(p_0-->q) & (p_1-->q) --> p_0 v p_1 --> q

Any occurrence of such a formula is pretty-printed as a --> b --> q.

NOTE
^^^^
- In a definition

   at = f

  f cannot contain (directly or indirectly) occurrences of at.

- The elements of list  must be separated by a comma,

****

To write a formula definition, it is convenient to use the following
pattern:

test(formLabel, formula, Defs) :-
	Defs = [
           at_1 = f_1,
	   at_2 = f_2,
	   ....
	   at_n = f_n
	]

Note that Defs is a Prolog variable
(hence, it must starts with an upper-case letter).


See the examples below.

*/



%%%  PROVABLE  FORMULAS

test(tertium,  p  v  ~ p ).  % formula without definitions


test(dummett, d , Defs ) :-
	Defs = [
	  a_0 = p --> q,
	  a_1 = q --> p,
	  d = a_0 v a_1
	].


test(doubleNeg,   ~ ~ p --> p ).  % formula without definitions



test(peirce, a --> p , Defs ) :-
	Defs = [
	    a =  (p --> q) --> p
	].

test(deMorg,   a --> b  , Defs ) :-
	Defs = [
	    a = ~( p & q),
	    b = ~p v ~q
	].


test(exAppB,  a --> r ,Defs) :-
   Defs = [
       a = b_0 & b_1,
       b_0 = ( p --> q) --> p,
       b_1 =  p --> r
  ].


%%%  Ex. 2.16 in [Gabbay&Olivetti:00]
test(gabbayOli1, 'A' --> a , Defs) :-
   Defs = [
       'A_0' =  c --> a,
       'A_1' = (c --> a) --> c,
       'A' = 'A_0' & 'A_1'
    ].

%%%  Ex. 2.17 in [Gabbay&Olivetti:00]
test(gabbayOli2 , g --> a, Defs) :-
  Defs =  [
      f_1 = (b_1 --> a) --> c,
      f_2 = (b_2 --> a) --> c,
      f_3 =  b_1 --> b_2  --> c,
      f =  f_1 & f_2  & f_3,
      g = f_1 & f_2  & f_3  & (c --> a )
  ].

%%%  Ex. 2.36 in [Gabbay&Olivetti:00]
test(gabbayOli3, a --> r, Defs) :-
  Defs =  [
      a = a_0 & a_1,
      a_0 = ( p --> q) --> p,
      a_1 = p --> r

  ].



%%%  Ex. 2.46 in [Gabbay&Olivetti:00]
test(gabbayOli4, f --> g, Defs) :-
  Defs =  [
      f = ( a --> b) --> b,
      g = ~a --> b
  ].



test(or1,  a --> b , Defs) :-
   Defs = [
       a = a_0 & a_1,
       a_0  = p_0 --> q,
       a_1 =  p_1 -->q,
       b =  p_0 v p_1 --> q,
       c = p_0 v p_1
   ].


test(or2,  a --> b , Defs) :-
   Defs = [
       a =  p v q --> r ,
       b  = p v ( q -->r)
   ].


test(or3,  p  & 'A' --> c, Defs ) :-
  Defs =   [
      'A' =  p --> 'B',
      'B' =  'B_0' & 'B_1',
      'B_0' =  a --> c,
      'B_1' = (b --> c) &( a v b),
      'C_0' = b --> c,
      'C_1' = a v b,
      h = p & 'A'
    ].


test(or4, e, Defs):-
	Defs = [
	    e = d_0 v d_1 v d_2 v d_3,
	    e_2 =   d_0 v d_1 v d_2,
	    e_1 =   d_0 v d_1,
	    d_0 = p & q,
	    d_1 = p & ~q,
	    d_2 = ~p & q,
	    d_3 = ~p & ~q
	].



%%%  NON PROVABLE FORMULAS


test(f0,  a --> r , Defs) :-
    Defs = [
	a = b0 & b1,
	b0 = ( p --> q) --> p,
	b1 =  p --> ~r
    ].

test(f1, (p & q ) v (~p & q) v (p & ~q) v (~p & ~a)).









