%################### TRANSLATION PHI : MJCR ---> MJ #################

:- module(translationPhi,
	  [phi/2
	  ]).

:- use_module(syntax).
:- use_module(basic,[
		  weak_tree_set/3,
		  weak_tree_form/3
	      ]).

%     SYNTAX
%
%   *  A marked formula is represented by the operator ^, e.g.
%      "D marked" is represented by the Prolog term
%
%            D^
%
%   *  A pattern
%
%             Gamma ; H '|>'
%
%    is represented by the Prolog term
%
%           Gamma @ H '|>'
%
%
%  * A ptree is an MJC-tree of the form
%
%     { Gamma @ foc(A) '|>' }
%		.
%		.
%	        .
%       Gamma @ foc(H)  '|>'
%
%   where:
%    -   the pattern Gamma @ foc(H)  '|>' is the root
%    -   the pattern Gamma @ foc(A) '|>'  is the only open leaf.
%    -   the displayed branch only contains patterns
%


%===========   AUXILIARY PREDICATES =================




%%%   list_union(+List, -Union)
%
% Given a list of list [L1, ..., LN],
% yields the union L1 U ... U LN
%


list_union(L,Union) :-
	flatten(L,L1),
	list_to_set(L1,Union).



%%%  mark(+Delta, -DeltaMarked)
%
%  DeltaMarked = { ^D | D in Delta }
%  ^D := D  marked

mark(Delta, DeltaMarked) :-
   findall( D^ , member(D,Delta), DeltaMarked).


%==============  PHI ==================

%%%  phi(+MJCR_Proof, -MJC_Proof)
%  MJCR_proof:  proof of a regular sequent
%
% By cases on the root rule of the MJCR_Proof



% rest
phi( proof(rest,  Gamma  ==> F @ D_U_Delta,[E0]), Proof) :- !,
	E0 = proof(_R , _ ==> D @  _, _Prems), % extract D
	mark(D_U_Delta, DMarked_U_DeltaMarked),
	union(DMarked_U_DeltaMarked, Gamma, DMarked_U_DeltaMarked_U_Gamma ),
        phi(E0,P0),
	weak_tree_form(P0,D^,WP0),  % add D^ to the contexts in P0
	Proof = proof(rc, DMarked_U_DeltaMarked_U_Gamma ==> F , [WP0]).

%lf0
phi(proof(lf0,  H_U_Gamma  ==> F @ Delta,[E0]), Proof) :- !,
	E0  = proof(_R, _ @ [] @ foc(H) =*=> lab(0) @ _@ _ ,_Prems), % extract H
	mark(Delta,DeltaMarked),
	union(DeltaMarked, H_U_Gamma,   DeltaMarked_U_H_U_Gamma),
        union(DeltaMarked_U_H_U_Gamma,[F^], FMarked_U_DeltaMarked_U_H_U_Gamma),
	%  FMarked_U_DeltaMarked_U_H_U_Gamma = H_U_Lambda
	PTreeLeaf = proof(leaf,  FMarked_U_DeltaMarked_U_H_U_Gamma @ foc(H) '|>' ,[]),
	phi_foc(E0,H,[PTreeLeaf],P0),
	Proof_LC0 = proof(lc0,  FMarked_U_DeltaMarked_U_H_U_Gamma ==>F , [P0]),
	Proof = proof(rc,  DeltaMarked_U_H_U_Gamma ==> F, [Proof_LC0]).


%lf1
phi(proof(lf1,  H_U_Gamma  ==> F @ Delta,[E0]), Proof) :- !,
	E0  = proof(_R, _ @ [] @ foc(H) =*=> lab(1) @ _@ _ ,_Prems), % extract H
	mark(Delta,DeltaMarked),
	union(DeltaMarked, H_U_Gamma, DeltaMarked_U_H_U_Gamma),
	union([F^], DeltaMarked_U_H_U_Gamma,  H_U_Lambda),
	%   H_U_Lambda = FMarked_U_DeltaMarked_U_H_U_Gamma
	PTreeLeaf =   proof(leaf,  H_U_Lambda  @ foc(H) '|>'  ,[]),
	phi_foc(E0,H,[PTreeLeaf],P0),
	Proof = proof(lc1, DeltaMarked_U_H_U_Gamma ==> F, [P0]).


%rlf (rest+lf0)
phi( proof(rlf,  H_U_Gamma  ==> F @ P_U_Delta,[E0]), Proof) :- !,
	E0  = proof(_R, _ @ [] @ foc(H) =*=> lab(0) @ P @ _ ,_Prems), % extract H and P
	mark(P_U_Delta,PMarked_U_DeltaMarked),
	union(PMarked_U_DeltaMarked,H_U_Gamma,PMarked_U_DeltaMarked_U_H_Gamma),
	union([F^], PMarked_U_DeltaMarked_U_H_Gamma,  H_U_Lambda ),
	%   H_U_Lambda = FMarked_U_DeltaMarked_U_H_U_Gamma
	PTreeLeaf =  proof(leaf, H_U_Lambda @ foc(H) '|>',[]),
	phi_foc(E0,H,[PTreeLeaf],P0),
	Proof_LC0 = proof(lc0,	 H_U_Lambda ==> P, [P0]),
	Proof = proof(rc, PMarked_U_DeltaMarked_U_H_Gamma ==> F , [Proof_LC0]).




%rAnd
phi( proof(rAnd,  Gamma  ==> A & B @ Delta,[E0,E1]), Proof) :- !,
	mark(Delta,DeltaMarked),
	union(DeltaMarked, Gamma,   DeltaMarked_U_Gamma),
	phi(E0,P0),
	phi(E1,P1),
	Proof = proof(rAnd,  DeltaMarked_U_Gamma ==> A & B, [P0,P1]).

% rOr
phi( proof(rOr,  Gamma  ==> A v B @ Delta,[E0]), Proof) :- !,
	mark(Delta,DeltaMarked),
	union(DeltaMarked, Gamma,   DeltaMarked_U_Gamma),
	phi(E0,P0),
	weak_tree_form(P0 , (A v B)^, WP0),  % add (A v B)^ to the contexts in P0
	Proof = proof(rOrp,  DeltaMarked_U_Gamma ==> A v B,[WP0]).


% rImpl
phi( proof(rImpl,  Gamma  ==> A --> B @ Delta,[E0]), Proof) :- !,
	mark(Delta,DeltaMarked),
	union(DeltaMarked,Gamma, DeltaMarked_U_Gamma),
	phi(E0,P0),
	Proof = proof(rImpl,  DeltaMarked_U_Gamma ==> A --> B, [P0]).


%%%%  phi_foc(+MJCR_Proof,+H, +MJC_PTree_List, -MJC_Proof)
%
%  * MJCR_Proof:  proof of a focused-sequent
%
%          Gamma @ Theta @ foc(A) =*=> lab(l) @	F @ Delta
%
%  * H: the ancestor of A
%
%  * MJC_PTree_List:
%     for every B in Theta U {A}, a ptree of the  kind
%
%     { H_U_Lambda @ foc(B) '|>' }
%		.
%		.
%	        .
%      H_U_Lambda @ foc(H)  '|>'
%
%   where
%
%          Lambda ::= F^ U Delta^ U Gamma
%
%
%
%  ASSUMPTION: the first tree in the list is the ptree where B = A
%  ^^^^^^^^^^^
%
% *  MJC_Proof: translation of  MJCR_Proof


% ax
phi_foc(proof(ax,  Gamma @ _Theta @ foc(F) =*=> lab(0) @ F@ Delta, []), H,_PTrees, Proof ) :- !,
	mark(Delta,DeltaMarked),
	list_union([ [H, F^], DeltaMarked, Gamma], H_U_Lambda	),
	Proof =   proof(ax, H_U_Lambda @ foc(F) =*=> lab(0) @ F,[]).

% lBot
phi_foc(proof(lBot,  Gamma @ _Theta @ foc(#f) =*=> lab(1) @ F@ Delta, []), H,_PTrees, Proof ) :- !,
	mark(Delta,DeltaMarked),
	list_union([ [H,F^], DeltaMarked, Gamma],   H_U_Lambda	),
	Proof =   proof(lBot,  H_U_Lambda  @ foc(#f) =*=> lab(1) @ F,[]).



% lAnd0
phi_foc(proof(lAnd0, Gamma @ _Theta @ foc(A0 & A1) =*=> lab(L) @ F @ Delta , [E0]), H, PTrees, Proof) :- !,
	mark(Delta,DeltaMarked),
	list_union( [[H,F^], DeltaMarked,Gamma],H_U_Lambda),
	PTrees = [ PTand |PTreesTheta],
	up_expand(lAnd0,PTand,[],PT0),
	up_expand(lAnd1,PTand,[],PT1),
	PTrees0 = [PT0,PT1 | PTreesTheta],
	phi_foc(E0, H, PTrees0, P0),
	Proof = proof(lAnd0,H_U_Lambda @ foc(A0 & A1) =*=> lab(L) @ F, [P0]).


% lAnd1
phi_foc(proof(lAnd1, Gamma @ _Theta @ foc(A0 & A1) =*=> lab(L) @ F @ Delta , [E1]), H, PTrees, Proof) :- !,
	mark(Delta,DeltaMarked),
	list_union( [[H,F^], DeltaMarked,Gamma], H_U_Lambda),
	PTrees = [ PTand |PTreesTheta],
	up_expand(lAnd0,PTand,[],PT0),
	up_expand(lAnd1,PTand,[],PT1),
	PTrees1 = [PT1,PT0 | PTreesTheta],
	phi_foc(E1, H, PTrees1, P1),
	Proof = proof(lAnd1,H_U_Lambda @ foc(A0 & A1) =*=> lab(L) @ F  ,[P1]).


%lImpl - empty Theta
phi_foc(proof(lImpl, Gamma @ [] @ foc(A -->B) =*=> lab(L) @ F @ Delta , [E0,E1]), H, PTrees, Proof) :- !,
	mark(Delta,DeltaMarked),
	list_union( [[H,F^], DeltaMarked,Gamma],H_U_Lambda),
	PTrees = [ Timpl |PTreesTheta],
	phi(E0,P0),
	weak_tree_form(P0,H,WP0),  % add H to the contexts in P0
	up_expand(lImpl, Timpl,	[WP0],PTB),
	PTrees1 = [PTB | PTreesTheta],
	phi_foc(E1,H,PTrees1, P1),
	Proof = proof(lImpl, H_U_Lambda @ foc(A-->B) =*=> lab(L) @ F,[WP0,P1]).


%lImpl - nonempty Theta
phi_foc(proof(lImpl, Gamma @ [_X |_Theta1] @ foc(A -->B) =*=> lab(L) @ F @ Delta , [E0,E1]), H, PTrees, Proof) :- !,
	mark(Delta,DeltaMarked),
	list_union( [[H,F^], DeltaMarked, Gamma],H_U_Lambda),
	PTrees = [ PTimpl | PTreesTheta],
	phi(E0,P0),
	append(PTreesTheta,[P0],SubstPrems),
        P_lImpl = proof(subst,H_U_Lambda ==> A, SubstPrems),
	up_expand(lImpl, PTimpl,[P_lImpl],PTB),
	PTrees1 = [PTB | PTreesTheta],
	phi_foc(E1,H,PTrees1, P1),
	Proof = proof(lImpl,H_U_Lambda @ foc(A-->B) =*=> lab(L) @ F,[P_lImpl,P1]).

% lOr - empty Theta
phi_foc(proof(lOr, Gamma @ [] @ foc(A0 v A1) =*=> lab(0) @ F @ Delta , [E0,D0]), H, _PTrees, Proof) :- !,
	mark(Delta,DeltaMarked),
	list_union( [[H,F^], DeltaMarked,Gamma], H_U_Lambda),
	% left
	phi(E0,P0),
	weak_tree_set(P0, [H,F^], WP0),  % add H and F^ to the contexts in P0
	%  right
	phi(D0,Q0),
	weak_tree_set(Q0,[H,F^],WQ0),   % add H and F^ to the contexts in Q0
        % proof
        Proof = proof(lOr, H_U_Lambda @ foc(A0 v A1) =*=> lab(0) @ F, [WP0,WQ0]).



% lOr - nonempty Theta
phi_foc(proof(lOr, Gamma @ [_X |_Theta1] @ foc(A0 v A1) =*=> lab(0) @ F @ Delta , [E0,D0]), H, PTrees, Proof) :- !,
	mark(Delta,DeltaMarked),
	list_union( [[H,F^], DeltaMarked,Gamma],H_U_Lambda),
	PTrees = [ _PTreeOr | PTreesTheta],
	% left
	list_union([[A0], H_U_Lambda], A0_U_H_U_Lambda),
	phi(E0,P0),
	weak_tree_form(P0,F^,WP0),  % F^ to the contexts in P0
	append(PTreesTheta,[WP0], SubstPrems0),
	P1 = proof(subst, A0_U_H_U_Lambda ==> F,SubstPrems0),
	%  right
	list_union([[A1], H_U_Lambda], A1_U_H_U_Lambda),
	phi(D0,Q0),
	weak_tree_form(Q0,F^,WQ0), % F^ to the contexts in Q0
	append(PTreesTheta,[WQ0], SubstPrems1),
	Q1 = proof(subst, A1_U_H_U_Lambda ==> F,SubstPrems1),
        % proof
        Proof = proof(lOr, H_U_Lambda @ foc(A0 v A1) =*=> lab(0) @ F, [P1,Q1]).




%%%%  up_expand(+Rule,+PTree,+AuxiliaryProofList,-Exp_ptree)
%  * PTree: a ptree  of the form
%
%    { sigma' = Gamma @ foc(B) '|>'  }
%		 .
%	         .
%		 .
%        Gamma @ foc(H) '|>'
%
%
% * Rule: one of the rules lAnd0, lAnd1, lImpl
%
% *  AuxiliaryProofList:
%    -    []	   if Rule is lAnd0 or lAnd1
%    -  [Pr_lImpl]  if Rule is lImpl, where Pr_lImp is an MJC-derivation
%
%
% * Exp_ptree: ptree obtained by (bottom-up) applying Rule
%     to the top-sequent sigma'
%    If Rule = lImpl, Pr_lImpl must be the left-premise of lImpl
%


up_expand(lAnd0,  proof(leaf, Gamma @ foc(A0 & A1) '|>', []) ,[],PTExp) :- !,
	PT0 = proof(leaf, Gamma @ foc(A0)  '|>', []),
	PTExp = proof(lAnd0, Gamma @ foc(A0 & A1) '|>' , [PT0]) .


up_expand(lAnd1, proof(leaf, Gamma @ foc(A0 & A1) '|>' , [])  ,[],PTExp) :- !,
	PT1 =  proof(leaf, Gamma @ foc(A1) '|>' , []) ,
	PTExp =  proof(lAnd1, Gamma @ foc(A0 & A1) '|>' , [PT1]).



up_expand(lImpl,proof(leaf, Gamma @ foc(A-->B) '|>' , []) ,[P_lImpl],PTExp) :- !,
	PT1 = proof(leaf, Gamma @ foc(B) '|>' , []),
	PTExp = proof(lImpl, Gamma @ foc(A-->B) '|>' , [P_lImpl,PT1]).


up_expand(Rule,proof(R, Gamma @ foc(A0 & A1) '|>' , [PT0]), AuxList,Exp_ptree) :- !,
	assertion( member(R,[lAnd0,lAnd1])),
	up_expand(Rule, PT0, AuxList,ExpPT0),
	Exp_ptree = proof( R, Gamma @ foc(A0 & A1) '|>' , [ExpPT0]).


up_expand(Rule,proof(lImpl, Gamma @ foc(A --> B) '|>'  , [PT0,PT1]), AuxList,Exp_ptree) :- !,
	up_expand(Rule, PT1, AuxList,ExpT1),
	Exp_ptree = proof( lImpl, Gamma @ foc(A-->B) '|>'  , [PT0,ExpT1]).
























